/**
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 */
package com.sunlabs.archsim.fleet.components.trunk;

import java.util.LinkedList;
import java.util.List;

import com.sunlabs.archsim.base.hierarchy.Entities;
import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.engine.Director;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.fleet.ships.testing.PayloadSink;
import com.sunlabs.archsim.fleet.ships.testing.PayloadSource;
import com.sunlabs.archsim.runner.OperationLoader;
import com.sunlabs.archsim.runner.Runner;

import junit.framework.TestCase;
import junit.textui.TestRunner;

/**
 * @author ib27688
 *
 */
public class TrunkTest extends TestCase {

	private PayloadSink dataSink;

	private LinkedList<Object> referenceData= new LinkedList<Object>();
	
	// used for manula test invocation	
	public static void main(String[] args) {
        // run the tests
        TestRunner.run(TrunkTest.class);
	} // main
	
	public void setUp() {
                // clear the data in the PayloadSource, if any from previous test runs
                PayloadSource.clear();
		// add data items for the first source
		for (int i= 0; i<10; i++) {
			Data d= new Data(i);
			Payload p= new Payload();
			p.data= d;
			PayloadSource.add("BypassSource", d);
			referenceData.add(p);
		}
		// add data items for the second source
		// note - the data in the reference list is sorted
		for (int i= 100; i<120; i++) {
			Data d= new Data(i);
			Payload p= new Payload();
			p.data= d;
			PayloadSource.add("FabricSource", d);
			referenceData.add(p);
		}

		
	}
	
	public void tearDown() {
		Director.getInstance().masterClear();
	}
	
	// first do the self check
	public void testSelfCheck() {
		Object tst=Entities.find("TrunkTest");
		CompositeEntity testGroup= (CompositeEntity)tst;
		assert(testGroup.selfCheck());		
	}	
	
	// test the operation of the trunk
	public void testTrunkOperation() {
		OperationLoader ldr= new OperationLoader();
		List ops= ldr.load("src/" 
				+ TrunkTest.class.getPackage().getName().replace('.', '/') 
				+ "/TrunkTestScript.xml");
		
		Runner r= new Runner();
		r.perform(ops);

		
		// get the sink
		dataSink= (PayloadSink)(Entities.find("DataSink"));
		
		// compare whether the received data is the same as the data sent
		assertEquals(referenceData.size(), dataSink.receivedData.size());
		for (Payload p: dataSink.receivedData) {
			assert(referenceData.contains(p));
		}
		
	} // testTrunkOperation
} // TrunkTest
